
package magercises.doublebuffer.solution;

// Copyright 1996, <a href=http://www.MageLang.com>MageLang Institute</a>.

import java.awt.*;

public class SimpleDrawingRegion extends Canvas {
	protected int drawingAreaWidth;
	protected int drawingAreaHeight;
	private java.applet.Applet applet;

	protected static Color backgroundColor = Color.white;
	protected static Color drawColor = Color.black;
	protected static Color dragColor = Color.red;

	// a double buffer to record old rectangles
	protected Image buffer;
	protected Graphics bufferGr;

	protected int downX, downY; // where the mouse goes down at start of draw
	protected int w=0,h=0;      // how wide is the rectangle?


	public SimpleDrawingRegion(java.applet.Applet applet, int w, int h) {
		this.applet = applet;
		drawingAreaWidth = w;
		drawingAreaHeight = h;
		resize(w,h);
		setBackground(backgroundColor);

		// create the drawing area
		buffer = applet.createImage(drawingAreaWidth, drawingAreaHeight);
		bufferGr = buffer.getGraphics();
		
		// make the double-buffer white so it is consistent across platforms
		bufferGr.setColor(Color.white);
		bufferGr.fillRect(0,0,drawingAreaWidth, drawingAreaHeight);
		// draw in black
		bufferGr.setColor(drawColor);
		repaint();
	}
	/** Clip the x,y coordinate to with [1,w-1],[1-h] */
	protected Point clip(int x, int y) {
		if ( x>=drawingAreaWidth ) x = drawingAreaWidth-2;
		else if ( x<1 ) x = 1;
		if ( y>=drawingAreaHeight ) y = drawingAreaHeight-2;
		else if ( y<1 ) y = 1;
		return new Point(x,y);
	}
	/** Set a corner of the bounding box enclosing tool */
	public boolean mouseDown(Event e, int x, int y) {
		downX = x;
		downY = y;
		w = h = 0;
		return true;        // say that we handled this event
	}
	/** Draw (rubberband) on screen buffer */
	public boolean mouseDrag(Event e, int x, int y) {
		int xorColor = backgroundColor.getRGB() ^ dragColor.getRGB();       
		Graphics g = getGraphics(); // graphics perspective on canvas
		g.setXORMode(new Color(xorColor));
		g.drawRect(downX, downY, w, h); // draw over old one to erase it

		// compute new w, h (clip so can't leave drawing region)
		Point clipped = clip(x,y);
		w = clipped.x-downX;
		h = clipped.y-downY;

		// can't have negative w,h; can only drag down,right
		if ( w<0 ) {
			w=0;
		}
		if ( h<0 ) {
			h=0;
		}

		// NO REPAINT, we use XORing to leave the background alone!
		g.drawRect(downX, downY, w, h); // draw new one at new location
		return true;
	}
	/** Draw the rectangle on the background buffer and repaint */
	public boolean mouseUp(Event e, int x, int y) {
		bufferGr.drawRect(downX, downY, w, h);
		repaint();
		return true;
	}
	/**Make the double-buffer appear in the applet window.
	 */
	public void paint(Graphics g) {
		g.drawImage(buffer, 0, 0, this);
		g.drawRect(0,0,drawingAreaWidth-1,drawingAreaHeight-1);
	}
	/** Normally update erases then calls paint.  Erasing is
	 * a waste of time as we paste a whole background image to screen.
	 */
	public void update(Graphics g) {
		paint(g);
	}
}